﻿using System;
using System.Diagnostics;
using System.Reflection;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Shell;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;

namespace VS2010Addin.SQLAzure2010Addin
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    ///
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the 
    /// IVsPackage interface and uses the registration attributes defined in the framework to 
    /// register itself and its components with the shell.
    /// </summary> 
    // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
    // a package.
    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the informations needed to show the this package
    // in the Help/About dialog of Visual Studio.
    [InstalledProductRegistration("#110", "#112", "0.2.3", IconResourceID = 400)]
    // This attribute is needed to let the shell know that this package exposes some menus.  
    [ProvideMenuResource("Menus.ctmenu", 1)]
    // This attribute registers a tool window exposed by this package. 
    [ProvideToolWindow(typeof(ExplorerToolWindow), Style = VsDockStyle.Tabbed, Orientation = ToolWindowOrientation.Left)]
    [ProvideToolWindow(typeof(TableDataWindow), Style = VsDockStyle.MDI, MultiInstances = true, Transient = true)]
    [ProvideToolWindow(typeof(SqlEditorWindow), Style = VsDockStyle.MDI, MultiInstances = true, Transient = true)]
    [Guid(GuidList.guidSQLAzureExplorerPkgString)]
    public sealed class SQLAzure2010AddinPackage : Package, IVsInstalledProduct
    {
        /// <summary>
        /// Default constructor of the package.
        /// Inside this method you can place any initialization code that does not require 
        /// any Visual Studio service because at this point the package object is created but 
        /// not sited yet inside Visual Studio environment. The place to do all the other 
        /// initialization is the Initialize method.
        /// </summary>
        public SQLAzure2010AddinPackage()
        {
            Debug.Print("SQLAzure2010AddinPackage()");
        }

        /// <summary>
        /// This function is called when the user clicks the menu item that shows the 
        /// tool window. See the Initialize method to see how the menu item is associated to 
        /// this function using the OleMenuCommandService service and the MenuCommand class.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CreateExplorerToolWindow(object sender, EventArgs e)
        {
            ToolWindowPane explorerWindow = FindToolWindow(typeof(ExplorerToolWindow), 0, true);
            if ((null == explorerWindow) || (null == explorerWindow.Frame))
            {
                throw new NotSupportedException(Resources.CanNotCreateWindow);
            }

            ShowWindow(explorerWindow);
        }

        /// <summary>
        /// Support method for finding an existing or creating a new tool window based on type and id.
        /// </summary>
        /// <typeparam name="T">type of MDI tool window</typeparam>
        /// <param name="id">id of tool window</param>
        /// <returns>the tool window pane</returns>
        public ToolWindowPane CreateWindow<T>(int id)
        {
            //find existing tool window based on id
            var window = FindToolWindow(typeof(T), id, false);

            if (window == null)
            {
                //create a new window with explicit tool window id
                window = FindToolWindow(typeof(T), id, true);
                if ((null == window) || (null == window.Frame))
                {
                    throw new NotSupportedException(Resources.CanNotCreateWindow);
                }
            }

            ShowWindow(window);
            return window;

        }

        private static void ShowWindow(ToolWindowPane window)
        {
            var windowFrame = (IVsWindowFrame)window.Frame;
            ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        #region Package Members

        /// <summary>
        /// Initialization of the package; this method is called right after the package is sited, so this is the place
        /// where you can put all the initilaization code that rely on services provided by VisualStudio.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            // Add our command handlers for menu (commands must exist in the .vsct file)
            var mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (null != mcs)
            {
                // Create the command for the menu item.
                var menuCommandID = new CommandID(GuidList.guidSQLAzureExplorerCmdSet, (int)PkgCmdIDList.cmdidMyCommand);
                var menuItem = new MenuCommand(CreateExplorerToolWindow, menuCommandID);
                mcs.AddCommand(menuItem);
                // Create the command for the tool window
                var toolwndCommandID = new CommandID(GuidList.guidSQLAzureExplorerCmdSet, (int)PkgCmdIDList.cmdidMyTool);
                var menuToolWin = new MenuCommand(CreateExplorerToolWindow, toolwndCommandID);
                mcs.AddCommand(menuToolWin);
            }
        }

        #endregion

        #region Splash and product detail function

        public int IdBmpSplash(out uint pIdBmp)
        {
            pIdBmp = 500;
            return VSConstants.S_OK;
        }

        public int IdIcoLogoForAboutbox(out uint pIdIco)
        {
            pIdIco = 600;
            return VSConstants.S_OK;
        }

        public int OfficialName(out string pbstrName)
        {
            pbstrName = "SQL Azure Explorer";
            return VSConstants.S_OK;
        }

        public int ProductDetails(out string pbstrProductDetails)
        {
            pbstrProductDetails = "SQL Azure Explorer Addin for Visual Studio 2010 Beta 1";
            return VSConstants.S_OK;
        }

        public int ProductID(out string pbstrPID)
        {
            pbstrPID = Assembly.GetExecutingAssembly().GetName().Version.ToString();
            return VSConstants.S_OK;
        }

        #endregion
    }
}
