﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Text;

namespace VS2010Addin.SQLAzure2010Addin.Data
{
    public class Repository : IRepository
    {
        private readonly string _serverName;
        private readonly string _login;
        private readonly string _password;

        private const string SelectUsers = "SELECT u.name , (CASE WHEN (r.principal_id IS NULL) THEN 'public' ELSE r.name END) AS role, u.default_schema_name, " +
                                            "u.principal_id, u.sid FROM   sys.database_principals AS u LEFT OUTER JOIN (sys.database_role_members AS m " +
                                            "INNER JOIN sys.database_principals AS r ON m.role_principal_id = r.principal_id) ON m.member_principal_id = u.principal_id " +
                                            "WHERE  u.type <> 'R'";

        public Repository(string serverName, string login, string password)
        {
            _serverName = serverName;
            _login = login;
            _password = password;
        }

        public IList<string> GetDatabaseList()
        {
            using (var conn = GetDatabaseConnection("master"))
            {
                return new SqlCommand("SELECT Name FROM SYS.DATABASES", conn)
                                       .Map(r => r.GetString(0)).ToList();
            }
        }

        public IList<string> GetTableList(string database)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("SELECT table_name as Name, table_schema as Owner FROM INFORMATION_SCHEMA.TABLES WHERE table_type = 'BASE TABLE'", conn)
                                       .Map(r => string.Format("{0}.{1}", r["Owner"] as string, r["Name"] as string)).ToList();
            }
        }

        public IList<string> GetViewList(string database)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("SELECT table_name as Name, table_schema as Owner FROM INFORMATION_SCHEMA.TABLES WHERE table_type = 'VIEW'", conn)
                                       .Map(r => string.Format("{0}.{1}", r["Owner"] as string, r["Name"] as string)).ToList();
            }
        }

        public IList<string> GetSpList(string database)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("SELECT specific_name as Name, specific_schema as Owner FROM INFORMATION_SCHEMA.ROUTINES WHERE routine_type = 'PROCEDURE'", conn)
                                       .Map(r => string.Format("{0}.{1}", r["Owner"] as string, r["Name"] as string)).ToList();
            }
        }

        public IList<string> GetFunctionList(string databaseName)
        {
            using (var conn = GetDatabaseConnection(databaseName))
            {
                return new SqlCommand("SELECT specific_name as Name, specific_schema as Owner FROM INFORMATION_SCHEMA.ROUTINES WHERE routine_type = 'FUNCTION'", conn)
                                       .Map(r => string.Format("{0}.{1}", r["Owner"] as string, r["Name"] as string)).ToList();
            }
        }

        public void CreateLogin(string login, string password)
        {
            ExecuteSql("CREATE LOGIN " + login + " WITH password='" + password + "'", "master");
        }

        public void DropLogin(string login)
        {
            ExecuteSql("DROP LOGIN " + login, "master");
        }

        public IList<SqlLogin> GetLoginList()
        {
            using (var conn = GetDatabaseConnection("master"))
            {
                return new SqlCommand("SELECT Name, principal_id, Sid FROM SYS.sql_logins", conn)
                    .Map( r => new SqlLogin
                                   {
                                       Name = r["Name"] as string, 
                                       PrincipalId = ((int?)r["principal_id"]).Value, 
                                       Sid = r["Sid"] as byte[]
                                   }).
                    ToList();
            }
        }

        public IList<string> GetRoleList(string database)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("exec sp_helprole", conn)
                                       .Map(r => r["RoleName"] as string).ToList();
            }
        }

        public IList<SqlUser> GetUserList(string database)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand(SelectUsers, conn)
                    .Map(r => new SqlUser
                    {
                        Name = r["Name"] as string,
                        Role = r["role"] as string,
                        DefaltSchema = r["default_schema_name"] as string,
                        PrincipalId = ((int?)r["principal_id"]).Value,
                        Sid = r["Sid"] as byte[],
                        Database = database
                    }).
                    ToList();
            }
        }

     
        public void DropUser(string database, string user)
        {
            ExecuteSql("DROP USER " + user, database);
        }

        public void CreateUser(string database, string user, string fromLogin)
        {
            ExecuteSql("CREATE USER " + user + " FROM LOGIN " + fromLogin, database);
        }

        public void AddRolesToUser(string database, string user, IList<string> roles)
        {
            foreach (var role in roles)
            {
                try
                {
                    ExecuteSql("EXEC sp_addrolemember '" + role + "', '" + user + "'", database);
                }
                catch (Exception)
                {
                    throw new ApplicationException("Adding role '" + role + "' to user '" + user + "'");
                }
            }
        }

        public DataSet ExecuteSql(string sql, string database)
        {
            var ds = new DataSet();
            using (var conn = GetDatabaseConnection(database))
            {
                conn.Open();
                using (var command = conn.CreateCommand())
                {
                    command.CommandText = sql;
                    var adapter = new SqlDataAdapter(command);
                    adapter.Fill(ds);
                }
            }

            return ds;
        }

        public IList<string> GetTableOrViewColumns(string database, string tableName)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("SELECT COLUMN_NAME as Name FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = '" + tableName + "' ORDER BY ORDINAL_POSITION ASC", conn)
                    .Map(r =>  r["Name"] as string ).ToList();
            }
        }

        public string GetObjectDefinition(string database, string objectName)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("SELECT OBJECT_DEFINITION(OBJECT_ID('" + objectName + "'))", conn)
                    .Map(r => r[0] as string).ToList()[0];
            }
        }

        public string GetCreateScript(string scriptname)
        {
            var resources = new System.Resources.ResourceManager(typeof(Scripts));
            return (string)resources.GetObject(scriptname);
        }

        public string GetTableDefinition(string database, string tableName)
        {
            var resources = new System.Resources.ResourceManager(typeof(Scripts));
            var script = (string)resources.GetObject("script_table");
            if (script != null) script = script.Replace("##TABLENAME##", tableName);
            var ds = ExecuteSql(script, database);
            if (ds != null && ds.Tables != null && ds.Tables.Count == 1 && ds.Tables[0].Rows != null)
            {
                var sb = new StringBuilder();
                foreach (DataRow row in ds.Tables[0].Rows)
                {
                    sb.Append(row[0] + " ");
                }
                return sb.ToString();
            }
            return "Sorry, CREATE TABLE script generation failed.";
        }

        public IList<string> GetParameters(string database, string procedure)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("select parameter_name as Name from information_schema.parameters where specific_name = '" + procedure + "' and parameter_mode = 'IN' order by ordinal_position", conn)
                    .Map(r => r[0] as string).ToList();
            }
        }

        public IList<string> GetPrimaryKeys(string database, string table)
        {
            using (var conn = GetDatabaseConnection(database))
            {
                return new SqlCommand("select column_name as Name from information_schema.table_constraints pk inner join information_schema.key_column_usage c on c.table_name = pk.table_name " +
                "and c.constraint_name = pk.constraint_name where pk.table_name = '" + table + "' and constraint_type = 'primary key'", conn)
                    .Map(r => r[0] as string).ToList();
            }
        }

        public void DropDatabase(string database)
        {
            ExecuteSql("drop database " + database, "master");
        }

        public void CreateDatabase(string database)
        {
            ExecuteSql("create database " + database, "master");
        }

        #region Get data/sets

        public DataSet GetTableData(string database, string table, int rows)
        {
            return ExecuteSql("SELECT TOP " + rows + " * FROM " + table, database);
        }

        #endregion

        #region Support methods

        public SqlConnection GetDatabaseConnection(string database)
        {
            var dbConnStr = new SqlConnectionStringBuilder
            {
                DataSource = _serverName,
                InitialCatalog = database,
                Encrypt = true,
                TrustServerCertificate = true,
                UserID = _login,
                Password = _password
            };
            return new SqlConnection(dbConnStr.ToString());
        }

        #endregion

    }
}