﻿using System;
using System.Data.SqlClient;
using System.Diagnostics;
using Microsoft.VisualStudio.TestTools.UnitTesting;

using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Tests.Properties;

namespace VS2010Addin.SQLAzure2010Addin.Tests
{
    /// <summary>
    /// Summary description for RepositoryTests
    /// </summary>
    [TestClass]
    public class RepositoryTests
    {
        private static readonly Repository Repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);

        [ClassInitialize]
        public static void ClassInitialize(TestContext testContext)
        {
            Assert.IsFalse(Settings.Default.ServerName == "servername",
                           "please create and include an app.config file in the test project with your personal SQL Azure credentials." +
                           "But don't add it to Source Control because it will contain your credentials!");

            //add a test database with stuff
            try { ExecuteSql("DROP DATABASE IntegrationTest", "master"); } catch { } 
            ExecuteSql("CREATE DATABASE IntegrationTest", "master");
            ExecuteSql("CREATE TABLE MyTable (id INT NOT NULL, CONSTRAINT pk_mytable PRIMARY KEY CLUSTERED (id ASC))", "IntegrationTest");
            ExecuteSql("CREATE VIEW MyView AS SELECT * FROM MyTable","IntegrationTest");
            ExecuteSql("CREATE PROCEDURE MyProc AS SELECT * FROM MyTable", "IntegrationTest");
            ExecuteSql("CREATE FUNCTION MyFunction (@id int) RETURNS TABLE AS RETURN SELECT * FROM MyTable WHERE id = @id", "IntegrationTest");
            try { ExecuteSql("DROP LOGIN IntegrationTestLogin", "master"); }
            catch { }
            try { ExecuteSql("DROP LOGIN IntegrationTestLogin2", "master"); }
            catch { }
            ExecuteSql("CREATE LOGIN IntegrationTestLogin WITH PASSWORD='Password123'", "master");
            ExecuteSql("INSERT INTO MyTable VALUES(1)", "IntegrationTest");
        }

        [ClassCleanup]
        public static void ClassCleanup()
        {
            ExecuteSql("DROP DATABASE IntegrationTest", "master");
            ExecuteSql("DROP LOGIN IntegrationTestLogin","master");
        }

        private static void ExecuteSql(string sql, string database)
        {
            using (var conn = Repository.GetDatabaseConnection(database))
            {
                conn.Open();
                new SqlCommand(sql, conn).ExecuteNonQuery();
            }
        }

      
        [TestMethod]
        public void ShouldGetListOfDatabases()
        {
            var list = Repository.GetDatabaseList();
            Assert.IsTrue(list.Count > 0);  
            Assert.IsTrue(list.Contains("IntegrationTest"));
        }

        [TestMethod]
        public void ShouldGetListOfTables()
        {
            var list = Repository.GetTableList("IntegrationTest");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Contains("dbo.MyTable"));
        }

        [TestMethod]
        public void ShouldGetListOfViews()
        {
            var list = Repository.GetViewList("IntegrationTest");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Contains("dbo.MyView"));
        }

        [TestMethod]
        public void ShouldGetListOfSPs()
        {
            var list = Repository.GetSpList("IntegrationTest");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Contains("dbo.MyProc"));
        }

        [TestMethod]
        public void ShouldGetListOfFunctions()
        {
            var list = Repository.GetFunctionList("IntegrationTest");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Contains("dbo.MyFunction"));
        }

        [TestMethod]
        public void ShouldCreateAndDeleteLogin()
        {
            Repository.CreateLogin("IntegrationTestLogin2", "Password123");
            var list = Repository.GetLoginList();
            Assert.IsTrue(list.Count > 0);
            Assert.IsTrue(list.Has(x => x.Name == "IntegrationTestLogin2"));
            Repository.DropLogin("IntegrationTestLogin2");
            list = Repository.GetLoginList();
            Assert.IsTrue(!list.Has(x => x.Name == "IntegrationTestLogin2"));
        }

        [TestMethod]
        public void ShouldGetListOfLogins()
        {
            var list = Repository.GetLoginList();
            Assert.IsTrue(list.Count > 0);
            Assert.IsTrue(list.Has(x => x.Name == "IntegrationTestLogin"));
        }

        [TestMethod]
        public void ShouldGetListOfRoles()
        {
            var list = Repository.GetRoleList("IntegrationTest");
            Assert.IsTrue(list.Count > 0);
            Assert.IsTrue(list.Has(x => x == "db_owner"));
        }

        [TestMethod]
        public void ShouldCreateAndDropUser()
        {
            Repository.CreateLogin("IntegrationTestLogin2", "Password123");
            Repository.CreateUser("IntegrationTest", "IntegrationTestUser", "IntegrationTestLogin2");
            var list = Repository.GetUserList("IntegrationTest");
            Assert.IsTrue(list.Count > 0);
            Assert.IsTrue(list.Has(x => x.Name == "IntegrationTestUser"));
            Repository.DropUser("IntegrationTest", "IntegrationTestUser");
            list = Repository.GetUserList("IntegrationTest");
            Assert.IsTrue(!list.Has(x => x.Name == "IntegrationTestUser"));
            Repository.DropLogin("IntegrationTestLogin2");
        }

        [TestMethod]
        public void ShouldGetTableData()
        {
            var dt = Repository.GetTableData("IntegrationTest", "MyTable", 1).Tables[0];
            Assert.IsNotNull(dt.Rows);
            if (dt.Rows != null) Assert.IsTrue(dt.Rows.Count == 1);
        }

        [TestMethod]
        public void ShouldGetListOfColumnsForTableOrView()
        {
            var list = Repository.GetTableOrViewColumns("IntegrationTest", "MyTable");
            Assert.IsTrue(list.Count > 0);
            Assert.IsTrue(list.Has(x => x == "id"));
        }

        [TestMethod]
        public void ShouldGetListOfPrimaryKeys()
        {
            var list = Repository.GetPrimaryKeys("IntegrationTest", "MyTable");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Has(x => x == "id"));
        }

        [TestMethod]
        public void ShouldGetListOfParameters()
        {
            var list = Repository.GetParameters("IntegrationTest", "MyFunction");
            Assert.IsTrue(list.Count == 1);
            Assert.IsTrue(list.Has(x => x == "@id"));
        }

        [TestMethod]
        public void ShouldGetObjectDef()
        {
            var def = Repository.GetObjectDefinition("IntegrationTest", "MyView");
            Assert.IsTrue(def.Length > 0);
            Console.WriteLine(def);
        }
    }
}
